Param(
  [string]$IncomingXml
)

######################################################################################
# 
# set_art_status.ps1 - "Set Artifact Status" v1.0 
# 
# Written by: Christian @ IntegrationAdmin.com (info@integrationadmin.com)
# 
# Description: Script for dynamicly settting various the statuses on BizTalk artifacts. 
# 
# For information, comments, and/or questions please visit my website at:
# http://www.integrationadmin.com
#
# Feel free to change this script in any way you. But please let this comment section remain, thank you!
#
######################################################################################


# functions for orchestrations
function orch-status ([string]$Name, [string]$Status)
{
	# MSBTS_Orchestration: http://msdn.microsoft.com/en-us/library/ee254133(v=bts.10).aspx
	try {
		$Msbts = get-wmiobject MSBTS_Orchestration `
		-namespace 'root\MicrosoftBizTalkServer' `
		-filter "Name='$Name'"

		If ($Msbts){
			switch ( $Status )
			{
			  'Enlisted' { [void]$Msbts.Enlist() }
			  'Started' { [void]$Msbts.Start() }
			  'Stopped' { [void]$Msbts.Stop() }
			  'Unenlisted' { [void]$Msbts.Unenlist() }
			  default {	log "Status $Status is not a valid status to set on $Type $Name! Valid status are: Started, Stopped, Enlisted, Unenlisted" "error"} 
			}

			log "Done attempting to set $Type ""$Name"" to status $Status."

			$Msbts = get-wmiobject MSBTS_Orchestration `
			-namespace 'root\MicrosoftBizTalkServer' `
			-filter "Name='$Name'"

			$CurrStatus = $Msbts.OrchestrationStatus

			switch ( $CurrStatus )
			{
			  "1" { $CurrStatus = "Unbound" }
			  "2" { $CurrStatus = "Bound" }
			  "3" { $CurrStatus = "Stopped" }
			  "4" { $CurrStatus = "Started" }
			}

			log "Current status of $Type ""$Name"" is now: $CurrStatus"
		} else {
			log "Unable to find $Type ""$Name""!" "error"
		}
	}
	catch {
		$ErrorMsg = "Error when trying to set status $Status for $Type ""$Name""`r`n`r`n"
		$ErrorMsg += [string]$error[0]
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
		log $ErrorMsg "error"
	}
}

# function for host instances
function hostinst-status ([string]$Name, [string]$Status)
{
	# MSBTS_HostInstance: http://msdn.microsoft.com/en-us/library/ee268275(v=bts.10).aspx
	try {
		$Msbts = get-wmiobject MSBTS_HostInstance `
		-namespace 'root\MicrosoftBizTalkServer' `
		-filter "HostName='$Name'"
		
		If ($Msbts){
			switch ( $Status )
			{
			  'Started' { [void]$Msbts.Start() }
			  'Stopped' { [void]$Msbts.Stop() }
			  default {	log "Status $Status is not a valid status to set on $Type $Name! Valid status are: Started, Stopped" "error"} 
			}
		   
			log "Done attempting to set $Type ""$Name"" to status $Status."
			
			$Msbts = get-wmiobject MSBTS_HostInstance `
			-namespace 'root\MicrosoftBizTalkServer' `
			-filter "HostName='$Name'"
			
			$CurrStatus = $Msbts.ServiceState
			
			switch ( $CurrStatus )
			{
			  "1" { $CurrStatus = "Unbound" }
			  "2" { $CurrStatus = "Bound" }
			  "3" { $CurrStatus = "Stopped" }
			  "4" { $CurrStatus = "Started" }
			}
			log "Current status of $Type ""$Name"" is now: $CurrStatus"
		} else {
			log "Unable to find $Type ""$Name""!" "error"
		}
	}
	catch {
		$ErrorMsg = "Error when trying to set status $Status for Orchestration ""$Name""`r`n`r`n"
		$ErrorMsg += [string]$error[0]
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
		log $ErrorMsg "error"
	}
}

# function for sendports
function sendport-status ([string]$Name, [string]$Status)
{
	# MSBTS_SendPort: http://msdn.microsoft.com/en-us/library/ee268427(v=bts.10).aspx
	try {
		$Msbts = get-wmiobject MSBTS_SendPort `
		-namespace 'root\MicrosoftBizTalkServer' `
		-filter "Name='$Name'"
		
		If ($Msbts){
			switch ( $Status )
			{
			  'Started' { [void]$Msbts.Start() }
			  'Stopped' { [void]$Msbts.Stop() }
			  'Enlisted' { [void]$Msbts.Enlist() }
			  'Unenlisted' { [void]$Msbts.UnEnlist() }
			  default {	log "Status $Status is not a valid status to set on $Type ""$Name""! Valid status are: Started, Stopped, Enlisted, Unenlisted" "error"}
			}

			log "Done attempting to set $Type ""$Name"" to status $Status."

			$Msbts = get-wmiobject MSBTS_SendPort `
			  -namespace 'root\MicrosoftBizTalkServer' `
			  -filter "Name='$Name'"
			 
			$CurrStatus = $Msbts.Status

			switch ( $CurrStatus )
			{
			  "1" { $CurrStatus = "Bound" }
			  "2" { $CurrStatus = "Stopped" }
			  "3" { $CurrStatus = "Started" }
			}
			log "Current status of $Type ""$Name"" is now: $CurrStatus"
		} else {
			log "Unable to find $Type ""$Name""!" "error"
		}
	}
	catch {
		$ErrorMsg = "Error when trying to set status $Status for SendPort ""$Name""`r`n`r`n"
		$ErrorMsg += [string]$error[0]
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
		log $ErrorMsg "error"
	}
}

# function for receivelocations
function receivelocation-status ([string]$Name, [string]$Status)
{
	# MSBTS_ReceiveLocation: http://msdn.microsoft.com/en-us/library/ee277482(v=bts.10).aspx
	try {
		$Msbts = get-wmiobject MSBTS_ReceiveLocation `
		-namespace 'root\MicrosoftBizTalkServer' `
		-filter "Name='$Name'"
		
		If ($Msbts){
			switch ( $Status )
			{
			  'Disabled' { [void]$Msbts.Disable() }
			  'Enabled' { [void]$Msbts.Enable() }
			  default {	log "Status $Status is not a valid status to set on $Type ""$Name""! Valid status are: Disabled, Enabled" "error"}
			}

			log "Done attempting to set $Type ""$Name"" to status $Status."

			$Msbts = get-wmiobject MSBTS_ReceiveLocation `
			  -namespace 'root\MicrosoftBizTalkServer' `
			  -filter "Name='$Name'"
			  
			$CurrStatus = $Msbts.IsDisabled

			switch ( $CurrStatus )
			{
			  $false { $CurrStatus = "Enabled" }
			  $true { $CurrStatus = "Disabled" }
			}
			log "Current status of $Type ""$Name"" is now: $CurrStatus"
		} else {
			log "Unable to find $Type ""$Name""!" "error"
		}
	}
	catch {
		$ErrorMsg = "Error when trying to set status $Status for ReceiveLocation ""$Name""`r`n`r`n"
		$ErrorMsg += [string]$error[0]
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
		log $ErrorMsg "error"
	}
}

# function that handles all the logging to ps promt as well as txtfiles in .\log
function log ([string]$Msg, [string]$IsErrMsg){
	if ($Msg) {
		# Create a timestamp
		$Now = Get-Date
		[string]$Now = $Now
		
		# If msg is of type error
		If ($IsErrMsg -eq "error"){
			# Combine timestamp, msgtype, and actual msg
			$Msg = $Now + " ERROR! " + $Msg
			# Show result in cmd, using a specific color
			Write-Host $Msg -foregroundcolor "Red"
		# Else msg is of type info
		} Else {
			# Combine timestamp, msgtype, and actual msg
			$Msg = $Now + " INFO: " + $Msg
			# Show result in cmd, using a specific color
			Write-Host $Msg -foregroundcolor "Green"
		}
		
		# Append the msg to file
		$Msg | Out-File -filepath $LogPath -append
	}
}

# function for parsing xml file
function parse-xml ([string]$XmlFile){
	try {
		[xml]$Xml = Get-Content $XmlFile
		$Xpath = "/BtArtifacts/BtArtifact"
		
		If ($Xml){
			$Xml | select-xml -xpath $Xpath | foreach {
				$Type = $_.node.ArtifactType
				$Name = $_.node.ArtifactName
				$Status = $_.node.SetToStatus
				
				log "Will now attempt to set $Type ""$Name"" to status $Status..."
				
				switch ($Type) 
				{ 
					"Orchestration" {
						orch-status $Name $Status
					}
					"SendPort" {
						sendport-status $Name $Status
					}
					"HostInst" {
						hostinst-status $Name $Status
					}
					"ReceiveLocation" {
						receivelocation-status $Name $Status
					}
					default {
						log "$Type is not a valid value! Must be one of the following: Orchestration, SendPort, or ReceiveLocation. Script will skip processing BtArtifact $Name" "error"
					}
				}
				log "Done handling $Type ""$Name""!"
				log $Divider
			}
		} else {
			log "$XmlFile does not exist or has no content!" "error"
		}
	}
	catch {
		$ErrorMsg = "Error when trying to parse XML file ""$$XmlFile""`r`n`r`n"
		$ErrorMsg += [string]$error[0]
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
		log $ErrorMsg "error"
	}
}

# main function
try {
	### START TO SET NAMES FOR LOG FILES ###
	$CurrDate = Get-Date
	$LogTimestamp = $CurrDate.Year.ToString() + $CurrDate.Month.ToString() + $CurrDate.Day.ToString() + "-" + $CurrDate.Hour.ToString() + "." + $CurrDate.Minute.ToString() + "." + $CurrDate.Second.ToString() + "_" + $CurrDate.millisecond.ToString()
	
	If ($IncomingXml){
		$LogName = $IncomingXml | % {$_ -match "\\(?!.*\\).*.xml" > $null; $matches[0]}
		$LogName = $LogName -replace "\\", ""
	} else {
		$LogName = "NoXmlFile"
	}
	
	# Path and name of where info event(s) are to be logged
	$script:LogPath = ".\logs\" + $LogTimestamp + "_" + $LogName + "_log.txt"
	# Path and name of where errors event(s) are to be logged
	# Dividing text to make output more reader friendly
	$script:Divider = "`r`n`r`n*************************************************************************`r`n"
	### DONE TO SETTING NAMES FOR LOG FILES ###
		
	# Set via WMI name of MgmtDbServer
	$script:mgmtDbServer = get-wmiobject MSBTS_GroupSetting -namespace root\MicrosoftBizTalkServer | select-object -expand MgmtDbServerName

	# Set via WMI name of MgmtDb
	$script:mgmtDbName = get-wmiobject MSBTS_GroupSetting -namespace root\MicrosoftBizTalkServer | select-object -expand MgmtDbName

	# Load BizTalk API
	[void] [System.reflection.Assembly]::LoadWithPartialName("Microsoft.BizTalk.ExplorerOM")
	$Catalog = New-Object Microsoft.BizTalk.ExplorerOM.BtsCatalogExplorer
	$script:Catalog.ConnectionString = "SERVER=$mgmtDbServer;DATABASE=$mgmtDbName;Integrated Security=SSPI"
	
	log "*** SCRIPT STARTED ***"
	log "Output will be written to log: $LogPath"
	
	If ($IncomingXml){
		log "Using Xml configuration file: $IncomingXml"
		log $Divider
		parse-xml $IncomingXml
	} else {
		$ErrMsg =  "No path to an XML configuration file were ever sent as a parameter when executing this script!"
		$ErrMsg += "In order to use this script you must set a path to an valid XML configuration file."
		$ErrMsg += "See this example on howto successfully execute this script:`r`n`r`n.\set_art_status.ps1 .\configs\template_config_file.xml`r`n`r`n"
		log $ErrMsg "error"
	}
	log "*** SCRIPT ENDED ***"
}
catch {
	$ErrorMsg = "Error when initiating script! Using Mngmtdb: $mgmtDbServer MgmtDbName: $mgmtDbName`r`n`r`n"
	$ErrorMsg += [string]$error[0]
	$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage
	log $ErrorMsg "error"
}

